/**
  ******************************************************************************
  * File Name          : main.c
  * Description        : Main program body
  ******************************************************************************
  * This notice applies to any and all portions of this file
  * that are not between comment pairs USER CODE BEGIN and
  * USER CODE END. Other portions of this file, whether
  * inserted by the user or by software development tools
  * are owned by their respective copyright owners.
  *
  * Copyright (c) 2017 STMicroelectronics International N.V.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted, provided that the following conditions are met:
  *
  * 1. Redistribution of source code must retain the above copyright notice,
  *    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  * 3. Neither the name of STMicroelectronics nor the names of other
  *    contributors to this software may be used to endorse or promote products
  *    derived from this software without specific written permission.
  * 4. This software, including modifications and/or derivative works of this
  *    software, must execute solely and exclusively on microcontroller or
  *    microprocessor devices manufactured by or for STMicroelectronics.
  * 5. Redistribution and use of this software other than as permitted under
  *    this license is void and will automatically terminate your rights under
  *    this license.
  *
  * THIS SOFTWARE IS PROVIDED BY STMICROELECTRONICS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
  * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT
  * SHALL STMICROELECTRONICS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32l0xx_hal.h"
#include "cmsis_os.h"
#include "task.h"

/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "hw.h"
#include "radio.h"
#include "timeServer.h"
#include "delay.h"
#include "low_power.h"
#include "vcom.h"

//Error Handle Macro
#define Error_Handler() _Error_Handler(__FILE__, __LINE__)

//LoRa Stack Definitions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#if defined( USE_BAND_868 )
#define RF_FREQUENCY                                868000000 // Hz
#elif defined( USE_BAND_915 )
#define RF_FREQUENCY                                915000000 // Hz
#else
    #error "Please define a frequency band in the compiler options."
#endif
//LoRa Modulation (Activation in compiler settings - Add USE_MODEM_LORA as symbol)
#define TX_OUTPUT_POWER                             14        // dBm
#if defined( USE_MODEM_LORA )
#define LORA_BANDWIDTH                              0         // [0: 125 kHz, 1: 250 kHz, 2: 500 kHz,3: Reserved]
#define LORA_SPREADING_FACTOR                       7         // [SF7..SF12]
#define LORA_CODINGRATE                             1         // [1: 4/5,2: 4/6,3: 4/7,4: 4/8]
#define LORA_PREAMBLE_LENGTH                        8         // Same for Tx and Rx
#define LORA_SYMBOL_TIMEOUT                         5         // Symbols
#define LORA_FIX_LENGTH_PAYLOAD_ON                  false
#define LORA_IQ_INVERSION_ON                        false
//FSK Modulation (Activation in compiler settings - Add USE_MODEM_FSK as symbol)
#elif defined( USE_MODEM_FSK )
#define FSK_FDEV                                    25e3      // Hz
#define FSK_DATARATE                                50e3      // bps
#define FSK_BANDWIDTH                               50e3      // Hz
#define FSK_AFC_BANDWIDTH                           83.333e3  // Hz
#define FSK_PREAMBLE_LENGTH                         5         // Same for Tx and Rx
#define FSK_FIX_LENGTH_PAYLOAD_ON                   false
#else
    #error "Please define a modem in the compiler options."
#endif

//State definition
typedef enum
{
    LOWPOWER,
    RX,
    RX_TIMEOUT,
    RX_ERROR,
    TX,
    TX_TIMEOUT,
}States_t;

#define RX_TIMEOUT_VALUE                            1000
#define BUFFER_SIZE                                 64 			// Define the payload size here
#define LED_PERIOD_MS               				200
#define LEDS_OFF   do{ \
                   LED_Off( LED_BLUE ) ;   \
                   LED_Off( LED_RED ) ;    \
                   LED_Off( LED_GREEN1 ) ; \
                   LED_Off( LED_GREEN2 ) ; \
                   } while(0) ;


const uint8_t PingMsg[] = "PING";								//message PING
const uint8_t PongMsg[] = "PONG";								//message PONG
uint16_t BufferSize = BUFFER_SIZE;								//buffer size
uint8_t Buffer[BUFFER_SIZE];									//create buffer of size buffer_size
States_t State = LOWPOWER;										//initial state LOWPOWER
int8_t RssiValue = 0;											//receive signal strength indication
int8_t SnrValue = 0;											//signal to noise ratio
static  TimerEvent_t timerLed;									//LED timer object
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

//Private variables ---------------------------------------------------------
osThreadId LEDBlinkingHandle;
osThreadId defaultTaskHandle;
osThreadId LoRaHandle;

//Private function prototypes -----------------------------------------------
void SystemClock_Config(void);							//Init SysClock
static void MX_GPIO_Init(void);							//Init GPIO
void StartDefaultTask(void const * argument);			//Default task (Thread)
void Start_LED_Blinking_task(void const* pvArg);		//LED blinking task (Thread)
void LoRaInit(void);									//LoRa init

//Private function prototypes LoRa ------------------------------------------
void OnTxDone(void);
void OnRxDone(uint8_t *payload, uint16_t size, int16_t rssi, int8_t snr);
void OnTxTimeout(void);
void OnRxTimeout(void);
void OnRxError( void );
static void OnledEvent(void);
void LoRaPingPong_task(void);

int main(void)
{
  // MCU Configuration----------------------------------------------------------


  HAL_Init();					//Reset of all peripherals, Initializes the Flash interface and the Systick
  SystemClock_Config();			//Configure the system clock

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */


  //MX_GPIO_Init();				//Initialize all configured peripherals

  DBG_Init( );				//from PingPong, initialize GPIOs
  HW_Init( );					//Initialize hardware/peripherals (HW_AdcInit(),Radio.IoInit(),HW_SPI_Init(),HW_RTC_Init(),vcom_Init())

  /* USER CODE BEGIN RTOS_MUTEX */
  /* add mutexes, ... */
  /* USER CODE END RTOS_MUTEX */

  /* USER CODE BEGIN RTOS_SEMAPHORES */
  /* add semaphores, ... */
  /* USER CODE END RTOS_SEMAPHORES */

  /* USER CODE BEGIN RTOS_TIMERS */
  /* start timers, add new ones, ... */
  /* USER CODE END RTOS_TIMERS */

  //Init LoRa Stack for PingPong before starting LoRa thread
  DISABLE_IRQ( );
  LoRaInit();
  ENABLE_IRQ( );



  //Create the thread(s) - definition and creation of defaultTask
  osThreadDef(defaultTask, StartDefaultTask, osPriorityNormal, 0, 128);
  defaultTaskHandle = osThreadCreate(osThread(defaultTask), NULL);

  //osThreadDef(BlinkingLED, Start_LED_Blinking_task,osPriorityNormal,0,configMINIMAL_STACK_SIZE);
  //LEDBlinkingHandle = osThreadCreate(osThread(BlinkingLED),NULL);

//osThreadDef(LoRaThread,LoRaPingPong_task,osPriorityNormal,0,configMINIMAL_STACK_SIZE);
 // LoRaHandle = osThreadCreate(osThread(LoRaThread),NULL);

  // USER CODE BEGIN RTOS_QUEUES
  // add queues, ...
  // USER CODE END RTOS_QUEUES

  // Start scheduler
  osKernelStart();

  // We should never get here as control is now taken by the scheduler

  // Infinite loop - USER CODE BEGIN WHILE
  while (1)
  {

  }
}

// System Clock Configuration (FreeRTOS)
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;

    //Configure the main internal regulator output voltage
      __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

    //Initializes the CPU, AHB and APB busses clocks
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = 16;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLLMUL_4;
  RCC_OscInitStruct.PLL.PLLDIV = RCC_PLLDIV_2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

    //Initializes the CPU, AHB and APB busses clocks
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    //Configure the Systick interrupt time
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    //Configure the Systick
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  // SysTick_IRQn interrupt configuration
  HAL_NVIC_SetPriority(SysTick_IRQn, 3, 0);
}

//SystemClock Config (PingPong)
/*void SystemClock_Config( void )
{
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};

  // Enable HSE Oscillator and Activate PLL with HSE as source
  RCC_OscInitStruct.OscillatorType      = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSEState            = RCC_HSE_OFF;
  RCC_OscInitStruct.HSIState            = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState        = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource       = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLMUL          = RCC_PLLMUL_6;
  RCC_OscInitStruct.PLL.PLLDIV          = RCC_PLLDIV_3;

  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  // Set Voltage scale1 as MCU will run at 32MHz
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

  // Poll VOSF bit of in PWR_CSR. Wait until it is reset to 0
  while (__HAL_PWR_GET_FLAG(PWR_FLAG_VOS) != RESET) {};

  // Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2 clocks dividers
  RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;
  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
  {
    Error_Handler();
  }
} */

/** Configure pins as
        * Analog
        * Input
        * Output
        * EVENT_OUT
        * EXTI
     PA2   ------> USART2_TX
     PA3   ------> USART2_RX
*/
static void MX_GPIO_Init(void)
{

  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin : B1_Pin */
  GPIO_InitStruct.Pin = B1_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(B1_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : USART_TX_Pin USART_RX_Pin */
  GPIO_InitStruct.Pin = USART_TX_Pin|USART_RX_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF4_USART2;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : LD2_Pin */
  GPIO_InitStruct.Pin = LD2_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(LD2_GPIO_Port, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/* StartDefaultTask function */
void StartDefaultTask(void const * argument)
{

  /* USER CODE BEGIN 5 */
  /* Infinite loop */
  for(;;)
  {
    osDelay(2000);
  }
  /* USER CODE END 5 */
}

/**
  * @brief  Period elapsed callback in non blocking mode
  * @note   This function is called  when TIM22 interrupt took place, inside
  * HAL_TIM_IRQHandler(). It makes a direct call to HAL_IncTick() to increment
  * a global variable "uwTick" used as application time base.
  * @param  htim : TIM handle
  * @retval None
  */
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
/* USER CODE BEGIN Callback 0 */

/* USER CODE END Callback 0 */
  if (htim->Instance == TIM21) {
    HAL_IncTick();
  }
/* USER CODE BEGIN Callback 1 */

/* USER CODE END Callback 1 */
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void _Error_Handler(char * file, int line)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while(1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

//User Code functions%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void Start_LED_Blinking_task(void const * pvArg)
{
	for( ;; )
	    {

	    	HAL_GPIO_TogglePin(GPIOA,GPIO_PIN_5);
	    	//HAL_Delay(2000);
	    	osDelay(3000);
	     	    }
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#ifdef USE_FULL_ASSERT

/**
   * @brief Reports the name of the source file and the source line number
   * where the assert_param error has occurred.
   * @param file: pointer to the source file name
   * @param line: assert_param error line source number
   * @retval None
   */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
    ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */

}

#endif

//LoRa functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void OnTxDone(void)
{
    Radio.Sleep( );
    State = TX;
    PRINTF("OnTxDone\n\r");
}

void OnRxDone( uint8_t *payload, uint16_t size, int16_t rssi, int8_t snr )
{
    Radio.Sleep( );
    BufferSize = size;
    memcpy( Buffer, payload, BufferSize );
    RssiValue = rssi;
    SnrValue = snr;
    State = RX;

    PRINTF("OnRxDone\n\r");
    PRINTF("RssiValue=%d dBm, SnrValue=%d\n\r", rssi, snr);
}

void OnTxTimeout( void )
{
    Radio.Sleep( );
    State = TX_TIMEOUT;

    PRINTF("OnTxTimeout\n\r");
}

void OnRxTimeout( void )
{
    Radio.Sleep( );
    State = RX_TIMEOUT;
    PRINTF("OnRxTimeout\n\r");
}

void OnRxError( void )
{
    Radio.Sleep( );
    State = RX_ERROR;
    PRINTF("OnRxError\n\r");
}

static void OnledEvent( void )
{
  LED_Toggle( LED_BLUE ) ;
  LED_Toggle( LED_RED1 ) ;
  LED_Toggle( LED_RED2 ) ;
  LED_Toggle( LED_GREEN ) ;

  TimerStart(&timerLed );
}

void LoRaPingPong_task(void)
{
	//LoRaInit();
	bool isMaster = true;
	uint8_t i;
	//Radio.Rx( RX_TIMEOUT_VALUE );
	while(1){
	    switch( State )
	    {
	    case RX:
	      if( isMaster == true )
	      {
	        if( BufferSize > 0 )
	        {
	          if( strncmp( ( const char* )Buffer, ( const char* )PongMsg, 4 ) == 0 )
	          {
	            //TimerStop(&timerLed );
	            //LED_Off( LED_BLUE);
	            //LED_Off( LED_GREEN ) ;
	            //LED_Off( LED_RED1 ) ;;
	            // Indicates on a LED that the received frame is a PONG
	            //LED_Toggle( LED_RED2 ) ;

	            // Send the next PING frame
	            Buffer[0] = 'P';
	            Buffer[1] = 'I';
	            Buffer[2] = 'N';
	            Buffer[3] = 'G';
	            // We fill the buffer with numbers for the payload
	            for( i = 4; i < BufferSize; i++ )
	            {
	              Buffer[i] = i - 4;
	            }
	            PRINTF("...PING\n\r");

	            //DelayMs( 1 );
	            osDelay(1);
	            Radio.Send( Buffer, BufferSize );
	            }
	            else if( strncmp( ( const char* )Buffer, ( const char* )PingMsg, 4 ) == 0 )
	            { // A master already exists then become a slave
	              isMaster = false;
	              //GpioWrite( &Led2, 1 ); // Set LED off
	              Radio.Rx( RX_TIMEOUT_VALUE );
	            }
	            else // valid reception but neither a PING or a PONG message
	            {    // Set device as master ans start again
	              isMaster = true;
	              Radio.Rx( RX_TIMEOUT_VALUE );
	            }
	          }
	        }
	        else
	        {
	          if( BufferSize > 0 )
	          {
	            if( strncmp( ( const char* )Buffer, ( const char* )PingMsg, 4 ) == 0 )
	            {
	              // Indicates on a LED that the received frame is a PING
	              //TimerStop(&timerLed );
	              //LED_Off( LED_RED1);
	              //LED_Off( LED_RED2 ) ;
	              //LED_Off( LED_GREEN ) ;
	              //LED_Toggle( LED_BLUE );

	              // Send the reply to the PONG string
	              Buffer[0] = 'P';
	              Buffer[1] = 'O';
	              Buffer[2] = 'N';
	              Buffer[3] = 'G';
	              // We fill the buffer with numbers for the payload
	              for( i = 4; i < BufferSize; i++ )
	              {
	                Buffer[i] = i - 4;
	              }
	              DelayMs( 1 );

	              Radio.Send( Buffer, BufferSize );
	              PRINTF("...PONG\n\r");
	            }
	            else // valid reception but not a PING as expected
	            {    // Set device as master and start again
	              isMaster = true;
	              Radio.Rx( RX_TIMEOUT_VALUE );
	            }
	         }
	      }
	      //State = LOWPOWER;
	      break;
	    case TX:
	      // Indicates on a LED that we have sent a PING [Master]
	      // Indicates on a LED that we have sent a PONG [Slave]
	      //GpioWrite( &Led2, GpioRead( &Led2 ) ^ 1 );
	      Radio.Rx( RX_TIMEOUT_VALUE );
	      //State = LOWPOWER;
	      break;
	    case RX_TIMEOUT:
	    case RX_ERROR:
	      if( isMaster == true )
	      {
	        // Send the next PING frame
	        Buffer[0] = 'P';
	        Buffer[1] = 'I';
	        Buffer[2] = 'N';
	        Buffer[3] = 'G';
	        for( i = 4; i < BufferSize; i++ )
	        {
	          Buffer[i] = i - 4;
	        }
	        //DelayMs( 1 );
	        osDelay(1);
	        Radio.Send( Buffer, BufferSize );
	      }
	      else
	      {
	        Radio.Rx( RX_TIMEOUT_VALUE );
	      }
	      //State = LOWPOWER;
	      break;
	    case TX_TIMEOUT:
	      Radio.Rx( RX_TIMEOUT_VALUE );
	      //State = LOWPOWER;
	      break;
	    case LOWPOWER:
	      default:
	            // Set low power
	      break;
	    }

	/*    DISABLE_IRQ( );
	    // if an interupt has occured after __disable_irq, it is kept pending and cortex will not enter low power anyway
	    if (State == LOWPOWER)
	    {
	#ifndef LOW_POWER_DISABLE
	      LowPower_Handler( );
	#endif
	    }
	    ENABLE_IRQ( );*/
}	//end while(1)
}

void LoRaInit(void)
{
	  //Led Timers
	  /*TimerInit(&timerLed, OnledEvent);
	  TimerSetValue( &timerLed, LED_PERIOD_MS);

	  TimerStart(&timerLed ); */

	  // Radio initialization
	  static RadioEvents_t RadioEvents;
	  RadioEvents.TxDone = OnTxDone;
	  RadioEvents.RxDone = OnRxDone;
	  RadioEvents.TxTimeout = OnTxTimeout;
	  RadioEvents.RxTimeout = OnRxTimeout;
	  RadioEvents.RxError = OnRxError;

	  Radio.Init( &RadioEvents );

	  Radio.SetChannel( RF_FREQUENCY );

	#if defined( USE_MODEM_LORA )

	  Radio.SetTxConfig( MODEM_LORA, TX_OUTPUT_POWER, 0, LORA_BANDWIDTH,
	                                 LORA_SPREADING_FACTOR, LORA_CODINGRATE,
	                                   LORA_PREAMBLE_LENGTH, LORA_FIX_LENGTH_PAYLOAD_ON,
	                                   true, 0, 0, LORA_IQ_INVERSION_ON, 3000000 );

	  Radio.SetRxConfig( MODEM_LORA, LORA_BANDWIDTH, LORA_SPREADING_FACTOR,
	                                   LORA_CODINGRATE, 0, LORA_PREAMBLE_LENGTH,
	                                   LORA_SYMBOL_TIMEOUT, LORA_FIX_LENGTH_PAYLOAD_ON,
	                                   0, true, 0, 0, LORA_IQ_INVERSION_ON, true );

	#elif defined( USE_MODEM_FSK )

	  Radio.SetTxConfig( MODEM_FSK, TX_OUTPUT_POWER, FSK_FDEV, 0,
	                                  FSK_DATARATE, 0,
	                                  FSK_PREAMBLE_LENGTH, FSK_FIX_LENGTH_PAYLOAD_ON,
	                                  true, 0, 0, 0, 3000000 );

	  Radio.SetRxConfig( MODEM_FSK, FSK_BANDWIDTH, FSK_DATARATE,
	                                  0, FSK_AFC_BANDWIDTH, FSK_PREAMBLE_LENGTH,
	                                  0, FSK_FIX_LENGTH_PAYLOAD_ON, 0, true,
	                                  0, 0,false, true );

	#else
	    #error "Please define a frequency band in the compiler options."
	#endif

	 Radio.Rx( RX_TIMEOUT_VALUE );
}
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
/**
  * @}
  */

/**
  * @}
*/

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
